import os
from datetime import datetime, timedelta

import pandas as pd
import tushare as ts
import yaml
from sqlalchemy import create_engine, Column, Integer, String, Float, text
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

# 模块级单例
_config = None
_engine = None
_Session = None
Base = declarative_base()


def load_config():
    global _config
    if _config is not None:
        return _config

    config_path = 'config.yaml'
    if not os.path.exists(config_path):
        config = {
            'tushare_token': 'your_token_here',
            'mysql': {
                'user': 'user',
                'password': 'password',
                'host': '127.0.0.1',
                'port': 3306,
                'database': 'tushare',
                'charset': 'utf8'
            },
            'openai_api': {
                'api_key': 'sk-your-api-key-here',
                'base_url': 'https://api.tu-zi.com/v1',
                'model': 'gpt-4'
            }
        }
        with open(config_path, 'w') as f:
            yaml.dump(config, f)
        print(f"请在 {config_path} 中填入您的 tushare token、MySQL 连接信息以及第三方 API 信息")
        exit(1)

    with open(config_path, 'r') as f:
        _config = yaml.safe_load(f)

    return _config

def get_engine():
    """获取单例数据库引擎"""
    global _engine
    if _engine is None:
        config = load_config()
        mysql = config['mysql']
        connection_string = f"mysql+pymysql://{mysql['user']}:{mysql['password']}@{mysql['host']}:{mysql['port']}/{mysql['database']}?charset={mysql['charset']}&use_unicode=1"
        _engine = create_engine(connection_string)

    return _engine

def get_trade_cal(start_date=None, end_date=None):
    """
    获取指定时间段内的交易日历

    参数：
    start_date (str): 开始日期，格式'YYYYMMDD'
    end_date (str): 结束日期，格式'YYYYMMDD'

    返回：
    pandas.DataFrame: 交易日历
    """
    if start_date is None:
        start_date = (datetime.now() - timedelta(days=30)).strftime('%Y%m%d')
    if end_date is None:
        end_date = datetime.now().strftime('%Y%m%d')

    pro = ts.pro_api()
    trade_cal_df = pro.trade_cal(exchange='', start_date=start_date, end_date=end_date)
    return trade_cal_df[trade_cal_df['is_open'] == 1]['cal_date'].tolist()

def get_db_engine():
    """获取SQLite数据库引擎，如果不存在则创建"""
    global _engine
    if _engine is not None:
        return _engine

    config = load_config()
    db_path = config.get('db_path', 'data/market_data.db')

    # 确保数据库目录存在
    os.makedirs(os.path.dirname(db_path), exist_ok=True)

    # 创建SQLite数据库引擎
    _engine = create_engine(f'sqlite:///{db_path}', echo=False)
    return _engine

def get_session():
    """获取数据库会话"""
    global _Session
    if _Session is None:
        _Session = sessionmaker(bind=get_db_engine())
    return _Session()

def init_db():
    """初始化数据库表结构"""
    Base.metadata.create_all(get_db_engine())

def get_existing_trade_dates(table_name):
    """
    从数据库中获取已有的交易日期

    参数:
    table_name (str): 数据表名称

    返回:
    set: 已存在于数据库中的交易日期集合
    """
    engine = get_db_engine()
    query = f"SELECT DISTINCT trade_date FROM {table_name}"

    try:
        # SQLAlchemy 2.0+ 版本的执行方式
        from sqlalchemy import text
        with engine.connect() as connection:
            result = connection.execute(text(query))
            return {row[0] for row in result}
    except Exception as e:
        print(f"获取已存在交易日期时出错: {e}")
        return set()


def load_df_from_db(table_name, conditions=None):
    """
    从数据库中加载数据

    参数:
    table_name (str): 表名
    conditions (str): 过滤条件，如 "trade_date > '20230101'"

    返回:
    pandas.DataFrame: 查询结果
    """
    engine = get_db_engine()
    query = f"SELECT * FROM {table_name}"

    if conditions:
        query += f" WHERE {conditions}"

    return pd.read_sql(query, engine)


def save_df_to_db(df, table_name, if_exists='append'):
    """
    保存DataFrame到数据库

    参数:
    df (pandas.DataFrame): 要保存的数据
    table_name (str): 表名
    if_exists (str): 如果表存在时的操作: 'fail', 'replace', 或 'append'
    """
    engine = get_db_engine()
    df.to_sql(table_name, engine, if_exists=if_exists, index=False)